@@ -33,20 +33,30 @@
 #include "i915_drm.h"
 
 typedef struct igt_spin {
-	unsigned int handle;
-	timer_t timer;
-	struct igt_list link;
+	struct igt_list_head link;
+
+	uint32_t handle;
+	uint32_t poll_handle;
+
+	uint32_t *batch;
 
 	uint32_t *condition;
 	uint32_t cmd_precondition;
 
+	uint32_t *poll;
+#define SPIN_POLL_START_IDX 0
+
+	struct timespec last_signal;
+	pthread_t timer_thread;
+	int timerfd;
+
 	int out_fence;
 	struct drm_i915_gem_exec_object2 obj[2];
 #define IGT_SPIN_BATCH   1
 	struct drm_i915_gem_execbuffer2 execbuf;
-	uint32_t poll_handle;
-	uint32_t *poll;
-#define SPIN_POLL_START_IDX 0
+
+	unsigned int flags;
+#define SPIN_CLFLUSH (1 << 0)
 } igt_spin_t;
 
 struct igt_spin_factory {
@@ -54,12 +64,18 @@ struct igt_spin_factory {
 	uint32_t dependency;
 	unsigned int engine;
 	unsigned int flags;
+	int fence;
 };
 
-#define IGT_SPIN_FENCE_OUT     (1 << 0)
-#define IGT_SPIN_POLL_RUN      (1 << 1)
-#define IGT_SPIN_FAST          (1 << 2)
-#define IGT_SPIN_NO_PREEMPTION (1 << 3)
+#define IGT_SPIN_FENCE_IN      (1 << 0)
+#define IGT_SPIN_FENCE_SUBMIT  (1 << 1)
+#define IGT_SPIN_FENCE_OUT     (1 << 2)
+#define IGT_SPIN_POLL_RUN      (1 << 3)
+#define IGT_SPIN_FAST          (1 << 4)
+#define IGT_SPIN_NO_PREEMPTION (1 << 5)
+#define IGT_SPIN_INVALID_CS    (1 << 6)
+#define IGT_SPIN_USERPTR       (1 << 7)
+#define IGT_SPIN_SOFTDEP       (1 << 8)
 
 igt_spin_t *
 __igt_spin_factory(int fd, const struct igt_spin_factory *opts);
@@ -94,6 +110,7 @@ static inline void igt_spin_busywait_until_started(igt_spin_t *spin)
 
 void igt_terminate_spins(void);
 void igt_unshare_spins(void);
+void igt_free_spins(int i915);
 
 enum igt_cork_type {
 	CORK_SYNC_FD = 1,
