@@ -32,16 +32,18 @@
 #include <fcntl.h>
 
 #include "drmtest.h"
+#include "i915/gem.h"
+#include "i915/gem_engine_topology.h"
 #include "igt_aux.h"
 #include "igt_core.h"
 #include "igt_gt.h"
+#include "igt_params.h"
 #include "igt_sysfs.h"
 #include "igt_debugfs.h"
 #include "ioctl_wrappers.h"
 #include "intel_reg.h"
 #include "intel_chipset.h"
 #include "igt_dummyload.h"
-#include "i915/gem_engine_topology.h"
 
 /**
  * SECTION:igt_gt
@@ -118,7 +120,7 @@ static void eat_error_state(int dev)
 void igt_require_hang_ring(int fd, int ring)
 {
 	if (!igt_check_boolean_env_var("IGT_HANG", true))
-		igt_skip("hang injection disabled by user");
+		igt_skip("hang injection disabled by user [IGT_HANG=0]\n");
 
 	gem_require_ring(fd, ring);
 	gem_context_require_bannable(fd);
@@ -162,6 +164,7 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 	struct drm_i915_gem_context_param param = {
 		.ctx_id = ctx,
 	};
+	int allow_reset;
 	unsigned ban;
 
 	/*
@@ -169,17 +172,11 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 	 * to recover from reset and for it to remain wedged. It's hard to
 	 * say even if we do hang/reset making the test suspect.
 	 */
-	igt_require_gem(fd);
-
 	if (!igt_check_boolean_env_var("IGT_HANG", true))
-		igt_skip("hang injection disabled by user");
+		igt_skip("hang injection disabled by user [IGT_HANG=0]\n");
 	gem_context_require_bannable(fd);
-	if (!igt_check_boolean_env_var("IGT_HANG_WITHOUT_RESET", false))
-		igt_require(has_gpu_reset(fd));
-
-	igt_require(igt_sysfs_set_parameter
-		    (fd, "reset", "%d", INT_MAX /* any reset method */));
 
+	allow_reset = 1;
 	if ((flags & HANG_ALLOW_CAPTURE) == 0) {
 		param.param = I915_CONTEXT_PARAM_NO_ERROR_CAPTURE;
 		param.value = 1;
@@ -188,7 +185,12 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 		 * the right one).
 		 */
 		__gem_context_set_param(fd, &param);
+		allow_reset = INT_MAX; /* any reset method */
 	}
+	igt_require(igt_params_set(fd, "reset", "%d", allow_reset));
+
+	if (!igt_check_boolean_env_var("IGT_HANG_WITHOUT_RESET", false))
+		igt_require(has_gpu_reset(fd));
 
 	ban = context_get_ban(fd, ctx);
 	if ((flags & HANG_ALLOW_BAN) == 0)
@@ -366,7 +368,8 @@ void igt_force_gpu_reset(int drm_fd)
 
 	dir = igt_debugfs_dir(drm_fd);
 
-	igt_sysfs_set(dir, "i915_wedged", "-1");
+	wedged = 0;
+	igt_sysfs_printf(dir, "i915_wedged", "%llu", -1ull);
 	igt_sysfs_scanf(dir, "i915_wedged", "%d", &wedged);
 
 	close(dir);
@@ -377,8 +380,7 @@ void igt_force_gpu_reset(int drm_fd)
 
 /* GPU abusers */
 static struct igt_helper_process hang_helper;
-static void __attribute__((noreturn))
-hang_helper_process(pid_t pid, int fd)
+__noreturn static void hang_helper_process(pid_t pid, int fd)
 {
 	while (1) {
 		if (kill(pid, 0)) /* Parent has died, so must we. */
@@ -546,23 +548,15 @@ unsigned intel_detect_and_clear_missed_interrupts(int fd)
 	return missed;
 }
 
-const struct intel_execution_engine intel_execution_engines[] = {
-	{ "default", NULL, 0, 0 },
-	{ "render", "rcs0", I915_EXEC_RENDER, 0 },
-	{ "bsd", "vcs0", I915_EXEC_BSD, 0 },
-	{ "bsd1", "vcs0", I915_EXEC_BSD, 1<<13 /*I915_EXEC_BSD_RING1*/ },
-	{ "bsd2", "vcs1", I915_EXEC_BSD, 2<<13 /*I915_EXEC_BSD_RING2*/ },
-	{ "blt", "bcs0", I915_EXEC_BLT, 0 },
-	{ "vebox", "vecs0", I915_EXEC_VEBOX, 0 },
-	{ NULL, 0, 0 }
-};
-
 bool gem_class_can_store_dword(int fd, int class)
 {
 	uint16_t devid = intel_get_drm_devid(fd);
 	const struct intel_device_info *info = intel_get_device_info(devid);
 	const int gen = ffs(info->gen);
 
+	if (info->gen == 0) /* unknown, assume it just works */
+		return true;
+
 	if (gen <= 2) /* requires physical addresses */
 		return false;
 
@@ -570,7 +564,7 @@ bool gem_class_can_store_dword(int fd, int class)
 		return false; /* only supports physical addresses */
 
 	if (gen == 6 && class == I915_ENGINE_CLASS_VIDEO)
-		return false;
+		return false; /* broken, unbelievably broken */
 
 	if (info->is_broadwater)
 		return false; /* Not sure yet... */
@@ -610,26 +604,3 @@ int gem_execbuf_flags_to_engine_class(unsigned int flags)
 		igt_assert(0);
 	}
 }
-
-bool gem_ring_is_physical_engine(int fd, unsigned ring)
-{
-	if (ring == I915_EXEC_DEFAULT)
-		return false;
-
-	/* BSD uses an extra flag to chose between aliasing modes */
-	if ((ring & 63) == I915_EXEC_BSD) {
-		bool explicit_bsd = ring & (3 << 13);
-		bool has_bsd2 = gem_has_bsd2(fd);
-		return explicit_bsd ? has_bsd2 : !has_bsd2;
-	}
-
-	return true;
-}
-
-bool gem_ring_has_physical_engine(int fd, unsigned ring)
-{
-	if (!gem_ring_is_physical_engine(fd, ring))
-		return false;
-
-	return gem_has_ring(fd, ring);
-}
