@@ -21,6 +21,7 @@
  * IN THE SOFTWARE.
  */
 
+#include <ctype.h>
 #include <signal.h>
 #include <errno.h>
 
@@ -28,6 +29,7 @@
 #include "igt_core.h"
 #include "igt_kmod.h"
 #include "igt_sysfs.h"
+#include "igt_taints.h"
 
 /**
  * SECTION:igt_kmod
@@ -153,6 +155,46 @@ static int modprobe(struct kmod_module *kmod, const char *options)
 					       NULL, NULL, NULL);
 }
 
+/**
+ * igt_kmod_has_param:
+ * @mod_name: The name of the module
+ * @param: The name of the parameter
+ *
+ * Returns: true if the module has the parameter, false otherwise.
+ */
+bool igt_kmod_has_param(const char *module_name, const char *param)
+{
+	struct kmod_module *kmod;
+	struct kmod_list *d, *pre;
+	bool result = false;
+
+	if (kmod_module_new_from_name(kmod_ctx(), module_name, &kmod))
+		return false;
+
+	pre = NULL;
+	if (!kmod_module_get_info(kmod, &pre))
+		goto out;
+
+	kmod_list_foreach(d, pre) {
+		const char *key, *val;
+
+		key = kmod_module_info_get_key(d);
+		if (strcmp(key, "parmtype"))
+			continue;
+
+		val = kmod_module_info_get_value(d);
+		if (val && strncmp(val, param, strlen(param)) == 0) {
+			result = true;
+			break;
+		}
+	}
+	kmod_module_info_free_list(pre);
+
+out:
+	kmod_module_unref(kmod);
+	return result;
+}
+
 /**
  * igt_kmod_load:
  * @mod_name: The name of the module
@@ -202,6 +244,25 @@ out:
 	return err < 0 ? err : 0;
 }
 
+static int igt_kmod_unload_r(struct kmod_module *kmod, unsigned int flags)
+{
+	struct kmod_list *holders, *pos;
+	int err = 0;
+
+	holders = kmod_module_get_holders(kmod);
+	kmod_list_foreach(pos, holders) {
+		struct kmod_module *it = kmod_module_get_module(pos);
+		err = igt_kmod_unload_r(it, flags);
+		kmod_module_unref(it);
+		if (err < 0)
+			break;
+	}
+	kmod_module_unref_list(holders);
+	if (err < 0)
+		return err;
+
+	return kmod_module_remove_module(kmod, flags);
+}
 
 /**
  * igt_kmod_unload:
@@ -228,7 +289,7 @@ igt_kmod_unload(const char *mod_name, unsigned int flags)
 		goto out;
 	}
 
-	err = kmod_module_remove_module(kmod, flags);
+	err = igt_kmod_unload_r(kmod, flags);
 	if (err < 0) {
 		igt_debug("Could not remove module %s (%s)\n", mod_name,
 			  strerror(-err));
@@ -345,7 +406,7 @@ igt_i915_driver_unload(void)
 		}
 	}
 
-	/* gen5 */
+	/* gen5: ips uses symbol_get() so only a soft module dependency */
 	if (igt_kmod_is_loaded("intel_ips"))
 		igt_kmod_unload("intel_ips", 0);
 
@@ -374,33 +435,49 @@ igt_i915_driver_unload(void)
 
 static void kmsg_dump(int fd)
 {
-	FILE *file;
+	char record[4096 + 1];
 
-	file = NULL;
-	if (fd != -1)
-		file = fdopen(fd, "r");
-	if (file) {
-		size_t len = 0;
-		char *line = NULL;
+	if (fd == -1) {
+		igt_warn("Unable to retrieve kernel log (from /dev/kmsg)\n");
+		return;
+	}
 
-		while (getline(&line, &len, file) != -1) {
-			char *start = strchr(line, ':');
-			if (start)
-				igt_warn("%s", start + 2);
+	record[sizeof(record) - 1] = '\0';
+
+	for (;;) {
+		const char *start, *end;
+		ssize_t r;
+
+		r = read(fd, record, sizeof(record) - 1);
+		if (r < 0) {
+			if (errno == EINTR)
+				continue;
+
+			if (errno == EPIPE) {
+				igt_warn("kmsg truncated: too many messages. You may want to increase log_buf_len in kmcdline\n");
+				continue;
+			}
+
+			if (errno != EAGAIN)
+				igt_warn("kmsg truncated: unknown error (%m)\n");
+
+			break;
 		}
 
-		free(line);
-		fclose(file);
-	} else {
-		igt_warn("Unable to retrieve kernel log (from /dev/kmsg)\n");
+		start = strchr(record, ';');
+		if (start) {
+			start++;
+			end = strchrnul(start, '\n');
+			igt_warn("%.*s\n", (int)(end - start), start);
+		}
 	}
 }
 
-static void tests_add(struct igt_kselftest_list *tl, struct igt_list *list)
+static void tests_add(struct igt_kselftest_list *tl, struct igt_list_head *list)
 {
 	struct igt_kselftest_list *pos;
 
-	igt_list_for_each(pos, list, link)
+	igt_list_for_each_entry(pos, list, link)
 		if (pos->number > tl->number)
 			break;
 
@@ -409,7 +486,7 @@ static void tests_add(struct igt_kselftest_list *tl, struct igt_list *list)
 
 void igt_kselftest_get_tests(struct kmod_module *kmod,
 			     const char *filter,
-			     struct igt_list *tests)
+			     struct igt_list_head *tests)
 {
 	const char *param_prefix = "igt__";
 	const int prefix_len = strlen(param_prefix);
@@ -506,9 +583,12 @@ int igt_kselftest_execute(struct igt_kselftest *tst,
 			  const char *options,
 			  const char *result)
 {
+	unsigned long taints;
 	char buf[1024];
 	int err;
 
+	igt_skip_on(igt_kernel_tainted(&taints));
+
 	lseek(tst->kmsg, 0, SEEK_END);
 
 	snprintf(buf, sizeof(buf), "%s=1 %s", tl->param, options ?: "");
@@ -531,6 +611,8 @@ int igt_kselftest_execute(struct igt_kselftest *tst,
 		     "kselftest \"%s %s\" failed: %s [%d]\n",
 		     tst->module_name, buf, strerror(-err), -err);
 
+	igt_assert_eq(igt_kernel_tainted(&taints), 0);
+
 	return err;
 }
 
@@ -546,13 +628,25 @@ void igt_kselftest_fini(struct igt_kselftest *tst)
 	kmod_module_unref(tst->kmod);
 }
 
+static const char *unfilter(const char *filter, const char *name)
+{
+	if (!filter)
+		return name;
+
+	name += strlen(filter);
+	if (!isalpha(*name))
+		name++;
+
+	return name;
+}
+
 void igt_kselftests(const char *module_name,
 		    const char *options,
 		    const char *result,
 		    const char *filter)
 {
 	struct igt_kselftest tst;
-	IGT_LIST(tests);
+	IGT_LIST_HEAD(tests);
 	struct igt_kselftest_list *tl, *tn;
 
 	if (igt_kselftest_init(&tst, module_name) != 0)
@@ -562,10 +656,19 @@ void igt_kselftests(const char *module_name,
 		igt_require(igt_kselftest_begin(&tst) == 0);
 
 	igt_kselftest_get_tests(tst.kmod, filter, &tests);
-	igt_list_for_each_safe(tl, tn, &tests, link) {
-		igt_subtest_f("%s", tl->name)
-			igt_kselftest_execute(&tst, tl, options, result);
-		free(tl);
+	igt_subtest_with_dynamic(filter ?: "all") {
+		igt_list_for_each_entry_safe(tl, tn, &tests, link) {
+			unsigned long taints;
+
+			igt_dynamic_f("%s", unfilter(filter, tl->name))
+				igt_kselftest_execute(&tst, tl, options, result);
+			free(tl);
+
+			if (igt_kernel_tainted(&taints)) {
+				igt_info("Kernel tainted, not executing more selftests.\n");
+				break;
+			}
+		}
 	}
 
 	igt_fixture {
