@@ -32,12 +32,15 @@
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 #include <sys/types.h>
 #include <dirent.h>
 
 #include "drmtest.h"
+#include "igt_kms.h"
 #include "igt_pm.h"
 #include "igt_aux.h"
+#include "igt_sysfs.h"
 
 /**
  * SECTION:igt_pm
@@ -58,16 +61,46 @@ enum {
 	POLICY_MIN_POWER = 2
 };
 
+#define MSR_PKG_CST_CONFIG_CONTROL	0xE2
+/*
+ * Below PKG CST limit mask and PC8 bits are meant for
+ * HSW,BDW SKL,ICL and Goldmont Microarch and future platforms.
+ * Refer IA S/W developers manual vol3c part3 chapter:35
+ */
+#define  PKG_CST_LIMIT_MASK		0xF
+#define  PKG_CST_LIMIT_C8		0x6
+
 #define MAX_PERFORMANCE_STR	"max_performance\n"
 #define MEDIUM_POWER_STR	"medium_power\n"
 #define MIN_POWER_STR		"min_power\n"
 /* Remember to fix this if adding longer strings */
 #define MAX_POLICY_STRLEN	strlen(MAX_PERFORMANCE_STR)
+int8_t *__sata_pm_policies;
+int __scsi_host_cnt;
+
+static int __igt_pm_power = -1;
 
 static char __igt_pm_audio_runtime_power_save[64];
 static char * __igt_pm_audio_runtime_control_path;
 static char __igt_pm_audio_runtime_control[64];
 
+static void __igt_pm_sata_link_pm_exit_handler(int sig);
+static void __igt_pm_restore_sata_link_power_management(void);
+
+static int find_runtime_pm(int device)
+{
+	char path[128];
+	struct stat st;
+
+	if (fstat(device, &st) || !S_ISCHR(st.st_mode))
+		return -1;
+
+	snprintf(path, sizeof(path), "/sys/dev/char/%d:%d/device/power",
+		 major(st.st_rdev), minor(st.st_rdev));
+
+	return open(path, O_RDONLY);
+}
+
 static int __igt_pm_audio_restore_runtime_pm(void)
 {
 	int fd;
@@ -280,39 +313,26 @@ void igt_pm_enable_audio_runtime_pm(void)
 		igt_debug("Failed to enable audio runtime PM! (%d)\n", -err);
 }
 
-/**
- * igt_pm_enable_sata_link_power_management:
- *
- * Enable the min_power policy for SATA link power management.
- * Without this we cannot reach deep runtime power states.
- *
- * We don't have any assertions on open since the system might not have
- * a SATA host.
- *
- * Returns:
- * An opaque pointer to the data needed to restore the default values
- * after the test has terminated, or NULL if SATA link power management
- * is not supported. This pointer should be freed when no longer used
- * (typically after having called restore_sata_link_power_management()).
- */
-int8_t *igt_pm_enable_sata_link_power_management(void)
+static void __igt_pm_enable_sata_link_power_management(void)
 {
 	int fd, i;
 	ssize_t len;
 	char *buf;
 	char *file_name;
-	int8_t *link_pm_policies = NULL;
+	int8_t policy;
 
 	file_name = malloc(PATH_MAX);
 	buf = malloc(MAX_POLICY_STRLEN + 1);
 
-	for (i = 0; ; i++) {
-		int8_t policy;
-
+	for (__scsi_host_cnt = 0; ; __scsi_host_cnt++) {
 		snprintf(file_name, PATH_MAX,
 			 "/sys/class/scsi_host/host%d/link_power_management_policy",
-			 i);
+			 __scsi_host_cnt);
 
+		/*
+		 * We don't have any assertions on open since the system
+		 * might not have a SATA host.
+		 */
 		fd = open(file_name, O_RDWR);
 		if (fd < 0)
 			break;
@@ -332,12 +352,26 @@ int8_t *igt_pm_enable_sata_link_power_management(void)
 		else
 			policy = POLICY_UNKNOWN;
 
-		if (!(i % 256))
-			link_pm_policies = realloc(link_pm_policies,
-						   (i / 256 + 1) * 256 + 1);
+		if (!(__scsi_host_cnt % 256))
+			__sata_pm_policies = realloc(__sata_pm_policies,
+						     (__scsi_host_cnt / 256 + 1)
+						     * 256 + 1);
 
-		link_pm_policies[i] = policy;
-		link_pm_policies[i + 1] = 0;
+		__sata_pm_policies[__scsi_host_cnt] = policy;
+		close(fd);
+	}
+
+	igt_install_exit_handler(__igt_pm_sata_link_pm_exit_handler);
+
+	for (i = 0; i < __scsi_host_cnt; i++) {
+		snprintf(file_name, PATH_MAX,
+			 "/sys/class/scsi_host/host%d/link_power_management_policy",
+			 i);
+		fd = open(file_name, O_RDWR);
+		if (fd < 0)
+			break;
+
+		policy = __sata_pm_policies[i];
 
 		/* If the policy is something we don't know about,
 		 * don't touch it, since we might potentially break things.
@@ -355,39 +389,25 @@ int8_t *igt_pm_enable_sata_link_power_management(void)
 	}
 	free(buf);
 	free(file_name);
-
-	return link_pm_policies;
 }
 
-/**
- * igt_pm_restore_sata_link_power_management:
- * @pm_data: An opaque pointer with saved link PM policies;
- *           If NULL is passed we force enable the "max_performance" policy.
- *
- * Restore the link power management policies to the values
- * prior to enabling min_power.
- *
- * Caveat: If the system supports hotplugging and hotplugging takes
- *         place during our testing so that the hosts change numbers
- *         we might restore the settings to the wrong hosts.
- */
-void igt_pm_restore_sata_link_power_management(int8_t *pm_data)
-
+static void __igt_pm_restore_sata_link_power_management(void)
 {
 	int fd, i;
 	char *file_name;
 
+	if (!__sata_pm_policies)
+		return;
+
 	/* Disk runtime PM policies. */
 	file_name = malloc(PATH_MAX);
-	for (i = 0; ; i++) {
+	for (i = 0; i < __scsi_host_cnt; i++) {
 		int8_t policy;
 
-		if (!pm_data)
-			policy = POLICY_MAX_PERFORMANCE;
-		else if (pm_data[i] == POLICY_UNKNOWN)
+		if (__sata_pm_policies[i] == POLICY_UNKNOWN)
 			continue;
 		else
-			policy = pm_data[i];
+			policy = __sata_pm_policies[i];
 
 		snprintf(file_name, PATH_MAX,
 			 "/sys/class/scsi_host/host%d/link_power_management_policy",
@@ -421,10 +441,47 @@ void igt_pm_restore_sata_link_power_management(int8_t *pm_data)
 		close(fd);
 	}
 	free(file_name);
+	free(__sata_pm_policies);
+	__sata_pm_policies = NULL;
+}
+
+/**
+ * igt_pm_enable_sata_link_power_management:
+ *
+ * Enable the min_power policy for SATA link power management.
+ * Without this we cannot reach deep runtime power states.
+ */
+void igt_pm_enable_sata_link_power_management(void)
+{
+	/* Check if has been already saved. */
+	if (__sata_pm_policies)
+		return;
+
+	 __igt_pm_enable_sata_link_power_management();
+}
+
+/**
+ * igt_pm_restore_sata_link_power_management:
+ *
+ * Restore the link power management policies to the values
+ * prior to enabling min_power.
+ *
+ * Caveat: If the system supports hotplugging and hotplugging takes
+ *         place during our testing so that the hosts change numbers
+ *         we might restore the settings to the wrong hosts.
+ */
+void igt_pm_restore_sata_link_power_management(void)
+{
+	if (!__sata_pm_policies)
+		return;
+
+	 __igt_pm_restore_sata_link_power_management();
+}
+
+static void __igt_pm_sata_link_pm_exit_handler(int sig)
+{
+	__igt_pm_restore_sata_link_power_management();
 }
-#define POWER_DIR "/sys/devices/pci0000:00/0000:00:02.0/power"
-/* We just leak this on exit ... */
-int pm_status_fd = -1;
 
 static char __igt_pm_runtime_autosuspend[64];
 static char __igt_pm_runtime_control[64];
@@ -433,10 +490,10 @@ static int __igt_restore_runtime_pm(void)
 {
 	int fd;
 
-	if (pm_status_fd < 0)
+	if (__igt_pm_power < 0)
 		return 0;
 
-	fd = open(POWER_DIR "/autosuspend_delay_ms", O_WRONLY);
+	fd = openat(__igt_pm_power, "autosuspend_delay_ms", O_WRONLY);
 	if (fd < 0)
 		return errno;
 
@@ -449,7 +506,7 @@ static int __igt_restore_runtime_pm(void)
 
 	close(fd);
 
-	fd = open(POWER_DIR "/control", O_WRONLY);
+	fd = openat(__igt_pm_power, "control", O_WRONLY);
 	if (fd < 0)
 		return errno;
 
@@ -462,8 +519,8 @@ static int __igt_restore_runtime_pm(void)
 
 	close(fd);
 
-	close(pm_status_fd);
-	pm_status_fd = -1;
+	close(__igt_pm_power);
+	__igt_pm_power = -1;
 
 	return 0;
 }
@@ -478,7 +535,7 @@ void igt_restore_runtime_pm(void)
 {
 	int ret;
 
-	if (pm_status_fd < 0)
+	if (__igt_pm_power < 0)
 		return;
 
 	igt_debug("Restoring runtime PM management to '%s' and '%s'\n",
@@ -506,15 +563,19 @@ static void __igt_pm_runtime_exit_handler(int sig)
  * Returns:
  * True if runtime pm is available, false otherwise.
  */
-bool igt_setup_runtime_pm(void)
+bool igt_setup_runtime_pm(int device)
 {
 	int fd;
 	ssize_t size;
 	char buf[6];
 
-	if (pm_status_fd >= 0)
+	if (__igt_pm_power != -1) /* XXX assume it's the same device! */
 		return true;
 
+	__igt_pm_power = find_runtime_pm(device);
+	if (__igt_pm_power < 0)
+		return false;
+
 	igt_pm_enable_audio_runtime_pm();
 
 	/*
@@ -522,9 +583,11 @@ bool igt_setup_runtime_pm(void)
 	 * test suite goes faster and we have a higher probability of
 	 * triggering race conditions.
 	 */
-	fd = open(POWER_DIR "/autosuspend_delay_ms", O_RDWR);
+	fd = openat(__igt_pm_power, "autosuspend_delay_ms", O_RDWR);
 	if (fd < 0) {
 		igt_pm_audio_restore_runtime_pm();
+		close(__igt_pm_power);
+		__igt_pm_power = -1;
 		return false;
 	}
 
@@ -542,6 +605,8 @@ bool igt_setup_runtime_pm(void)
 	if (size <= 0) {
 		close(fd);
 		igt_pm_audio_restore_runtime_pm();
+		close(__igt_pm_power);
+		__igt_pm_power = -1;
 		return false;
 	}
 
@@ -554,12 +619,15 @@ bool igt_setup_runtime_pm(void)
 
 	close(fd);
 
-	if (size != 2)
+	if (size != 2) {
+		close(__igt_pm_power);
+		__igt_pm_power = -1;
 		return false;
+	}
 
 	/* We know we support runtime PM, let's try to enable it now. */
-	fd = open(POWER_DIR "/control", O_RDWR);
-	igt_assert_f(fd >= 0, "Can't open " POWER_DIR "/control\n");
+	fd = openat(__igt_pm_power, "control", O_RDWR);
+	igt_assert_f(fd >= 0, "Can't open control\n");
 
 	igt_assert(read(fd, __igt_pm_runtime_control,
 			sizeof(__igt_pm_runtime_control) - 1) > 0);
@@ -578,25 +646,49 @@ bool igt_setup_runtime_pm(void)
 
 	close(fd);
 
-	pm_status_fd = open(POWER_DIR "/runtime_status", O_RDONLY);
-	igt_assert_f(pm_status_fd >= 0,
-		     "Can't open " POWER_DIR "/runtime_status\n");
-
 	return true;
 }
 
+/**
+ * igt_disable_runtime_pm:
+ *
+ * Disable the runtime pm for i915 device.
+ * igt_disable_runtime_pm assumes that igt_setup_runtime_pm has already
+ * called to save runtime autosuspend and control attributes.
+ */
+void igt_disable_runtime_pm(void)
+{
+	int fd;
+	ssize_t size;
+	char buf[6];
+
+	igt_assert_fd(__igt_pm_power);
+
+	/* We know we support runtime PM, let's try to disable it now. */
+	fd = openat(__igt_pm_power, "control", O_RDWR);
+	igt_assert_f(fd >= 0, "Can't open control\n");
+
+	size = write(fd, "on\n", 3);
+	igt_assert(size == 3);
+	lseek(fd, 0, SEEK_SET);
+	size = read(fd, buf, ARRAY_SIZE(buf));
+	igt_assert(size == 3);
+	igt_assert(strncmp(buf, "on\n", 3) == 0);
+	close(fd);
+}
+
 /**
  * igt_get_runtime_pm_status:
  *
  * Returns: The current runtime PM status.
  */
-enum igt_runtime_pm_status igt_get_runtime_pm_status(void)
+static enum igt_runtime_pm_status __igt_get_runtime_pm_status(int fd)
 {
 	ssize_t n_read;
 	char buf[32];
 
-	lseek(pm_status_fd, 0, SEEK_SET);
-	n_read = read(pm_status_fd, buf, ARRAY_SIZE(buf) - 1);
+	lseek(fd, 0, SEEK_SET);
+	n_read = read(fd, buf, ARRAY_SIZE(buf) - 1);
 	igt_assert(n_read >= 0);
 	buf[n_read] = '\0';
 
@@ -613,6 +705,45 @@ enum igt_runtime_pm_status igt_get_runtime_pm_status(void)
 	return IGT_RUNTIME_PM_STATUS_UNKNOWN;
 }
 
+enum igt_runtime_pm_status igt_get_runtime_pm_status(void)
+{
+	enum igt_runtime_pm_status status;
+	int fd;
+
+	if (__igt_pm_power < 0)
+		return IGT_RUNTIME_PM_STATUS_UNKNOWN;
+
+	fd = openat(__igt_pm_power, "runtime_status", O_RDONLY);
+	igt_assert_f(fd >= 0, "Can't open runtime_status\n");
+
+	status = __igt_get_runtime_pm_status(fd);
+	close(fd);
+
+	return status;
+}
+
+/**
+ * _pm_status_name
+ * @status: runtime PM status to stringify
+ *
+ * Returns: The current runtime PM status as a string
+ */
+static const char *_pm_status_name(enum igt_runtime_pm_status status)
+{
+	switch (status) {
+	case IGT_RUNTIME_PM_STATUS_ACTIVE:
+		return "active";
+	case IGT_RUNTIME_PM_STATUS_RESUMING:
+		return "resuming";
+	case IGT_RUNTIME_PM_STATUS_SUSPENDED:
+		return "suspended";
+	case IGT_RUNTIME_PM_STATUS_SUSPENDING:
+		return "suspending";
+	default:
+		return "unknown";
+	}
+}
+
 /**
  * igt_wait_for_pm_status:
  * @status: desired runtime PM status
@@ -626,5 +757,102 @@ enum igt_runtime_pm_status igt_get_runtime_pm_status(void)
  */
 bool igt_wait_for_pm_status(enum igt_runtime_pm_status status)
 {
-	return igt_wait(igt_get_runtime_pm_status() == status, 10000, 100);
+	enum igt_runtime_pm_status expected = status;
+	bool ret;
+	int fd;
+
+	if (__igt_pm_power < 0)
+		return false;
+
+	fd = openat(__igt_pm_power, "runtime_status", O_RDONLY);
+	igt_assert_f(fd >= 0, "Can't open runtime_status\n");
+
+	ret = igt_wait((status = __igt_get_runtime_pm_status(fd)) == expected,
+		       10000, 100);
+	close(fd);
+
+	if (!ret)
+		igt_warn("timeout: pm_status expected:%s, got:%s\n",
+			_pm_status_name(expected),
+			_pm_status_name(status));
+
+	return ret;
+}
+
+/**
+ * dmc_loaded:
+ * @debugfs: fd to the debugfs dir.
+ *
+ * Check whether DMC FW is loaded or not. DMC FW is require for few Display C
+ * states like DC5 and DC6. FW does the Context Save and Restore during Display
+ * C States entry and exit.
+ *
+ * Returns:
+ * True if DMC FW is loaded otherwise false.
+ */
+bool igt_pm_dmc_loaded(int debugfs)
+{
+	char buf[15];
+	int len;
+
+	len = igt_sysfs_read(debugfs, "i915_dmc_info", buf, sizeof(buf) - 1);
+	if (len < 0)
+		return true; /* no CSR support, no DMC requirement */
+
+	buf[len] = '\0';
+
+	igt_info("DMC: %s\n", buf);
+	return strstr(buf, "fw loaded: yes");
+}
+
+/**
+ * igt_pm_pc8_plus_residencies_enabled:
+ * @msr_fd: fd to /dev/cpu/0/msr
+ * Check whether BIOS has disabled the PC8 package deeper state.
+ *
+ * Returns:
+ * True if PC8+ package deeper state enabled on machine otherwise false.
+ */
+bool igt_pm_pc8_plus_residencies_enabled(int msr_fd)
+{
+	int rc;
+	uint64_t val;
+
+	rc = pread(msr_fd, &val, sizeof(uint64_t), MSR_PKG_CST_CONFIG_CONTROL);
+	if (rc != sizeof(val))
+		return false;
+	if ((val & PKG_CST_LIMIT_MASK) < PKG_CST_LIMIT_C8) {
+		igt_info("PKG C-states limited below PC8 by the BIOS\n");
+		return false;
+	}
+
+	return true;
+}
+
+/**
+ * i915_output_is_lpsp_capable:
+ * @drm_fd: fd to drm device
+ * @output: igt output for which lpsp capability need to be evaluated
+ * Check lpsp capability for a given output.
+ *
+ * Returns:
+ * True if given output is lpsp capable otherwise false.
+ */
+bool i915_output_is_lpsp_capable(int drm_fd, igt_output_t *output)
+{
+	char buf[256];
+	int fd, len;
+
+	fd = igt_debugfs_connector_dir(drm_fd, output->name, O_RDONLY);
+	igt_require(fd >= 0);
+	len = igt_debugfs_simple_read(fd, "i915_lpsp_capability",
+				      buf, sizeof(buf));
+
+	/* if i915_lpsp_capability not present return the capability as false */
+	if (len < 0)
+		return false;
+
+	close(fd);
+
+	return strstr(buf, "LPSP: capable");
 }
