@@ -1,4 +1,5 @@
 #include "settings.h"
+#include "version.h"
 
 #include <ctype.h>
 #include <errno.h>
@@ -11,15 +12,19 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/utsname.h>
 #include <unistd.h>
 
 enum {
 	OPT_ABORT_ON_ERROR,
+	OPT_DISK_USAGE_LIMIT,
 	OPT_TEST_LIST,
 	OPT_IGNORE_MISSING,
 	OPT_PIGLIT_DMESG,
 	OPT_DMESG_WARN_LEVEL,
 	OPT_OVERALL_TIMEOUT,
+	OPT_PER_TEST_TIMEOUT,
+	OPT_VERSION,
 	OPT_HELP = 'h',
 	OPT_NAME = 'n',
 	OPT_DRY_RUN = 'd',
@@ -51,6 +56,7 @@ static struct {
 } abort_conditions[] = {
 	{ ABORT_TAINT, "taint" },
 	{ ABORT_LOCKDEP, "lockdep" },
+	{ ABORT_PING, "ping" },
 	{ ABORT_ALL, "all" },
 	{ 0, 0 },
 };
@@ -119,6 +125,46 @@ static bool parse_abort_conditions(struct settings *settings, const char *optarg
 	return true;
 }
 
+static size_t char_to_multiplier(char c)
+{
+	switch (c) {
+	case 'k':
+	case 'K':
+		return 1024UL;
+	case 'm':
+	case 'M':
+		return 1024UL * 1024UL;
+	case 'g':
+	case 'G':
+		return 1024UL * 1024UL * 1024UL;
+	}
+
+	return 0;
+}
+
+static bool parse_usage_limit(struct settings *settings, const char *optarg)
+{
+	size_t value;
+	char *endptr = NULL;
+
+	if (!optarg)
+		return false;
+
+	value = strtoul(optarg, &endptr, 10);
+
+	if (*endptr) {
+		size_t multiplier = char_to_multiplier(*endptr);
+
+		if (multiplier == 0)
+			return false;
+
+		value *= multiplier;
+	}
+
+	settings->disk_usage_limit = value;
+	return true;
+}
+
 static const char *usage_str =
 	"usage: runner [options] [test_root] results-path\n"
 	"   or: runner --list-all [options] [test_root]\n\n"
@@ -140,6 +186,9 @@ static const char *usage_str =
 	"                        Possible conditions:\n"
 	"                         lockdep - abort when kernel lockdep has been angered.\n"
 	"                         taint   - abort when kernel becomes fatally tainted.\n"
+	"                         ping    - abort when a host configured in .igtrc or\n"
+	"                                   environment variable IGT_PING_HOSTNAME does\n"
+	"                                   not respond to ping.\n"
 	"                         all     - abort for all of the above.\n"
 	"  -s, --sync            Sync results to disk after every test\n"
 	"  -l {quiet,verbose,dummy}, --log-level {quiet,verbose,dummy}\n"
@@ -159,8 +208,17 @@ static const char *usage_str =
 	"  --inactivity-timeout <seconds>\n"
 	"                        Kill the running test after <seconds> of inactivity in\n"
 	"                        the test's stdout, stderr, or dmesg\n"
+	"  --per-test-timeout <seconds>\n"
+	"                        Kill the running test after <seconds>. This timeout is per\n"
+	"                        subtest, or dynamic subtest. In other words, every subtest,\n"
+	"                        even when running in multiple-mode, must finish in <seconds>.\n"
 	"  --overall-timeout <seconds>\n"
 	"                        Don't execute more tests after <seconds> has elapsed\n"
+	"  --disk-usage-limit <limit>\n"
+	"                        Kill the running test if its logging, both itself and the\n"
+	"                        kernel logs, exceed the given limit in bytes. The limit\n"
+	"                        parameter can use suffixes k, M and G for kilo/mega/gigabytes,\n"
+	"                        respectively. Limit of 0 (default) disables the limit.\n"
 	"  --use-watchdog        Use hardware watchdog for lethal enforcement of the\n"
 	"                        above timeout. Killing the test process is still\n"
 	"                        attempted at timeout trigger.\n"
@@ -227,7 +285,7 @@ static bool parse_blacklist(struct regex_list *exclude_regexes,
 	FILE *f;
 	char *line = NULL;
 	size_t line_len = 0;
-	bool status;
+	bool status = false;
 
 	if ((f = fopen(blacklist_filename, "r")) == NULL) {
 		fprintf(stderr, "Cannot open blacklist file %s\n", blacklist_filename);
@@ -283,6 +341,17 @@ static bool readable_file(char *filename)
 	return !access(filename, R_OK);
 }
 
+static void print_version(void)
+{
+	struct utsname uts;
+
+	uname(&uts);
+
+	printf("IGT-Version: %s-%s (%s) (%s: %s %s)\n", PACKAGE_VERSION,
+	       IGT_GIT_SHA1, TARGET_CPU_PLATFORM,
+	       uts.sysname, uts.release, uts.machine);
+}
+
 void init_settings(struct settings *settings)
 {
 	memset(settings, 0, sizeof(*settings));
@@ -308,12 +377,14 @@ bool parse_options(int argc, char **argv,
 	char *env_test_root;
 
 	static struct option long_options[] = {
+		{"version", no_argument, NULL, OPT_VERSION},
 		{"help", no_argument, NULL, OPT_HELP},
 		{"name", required_argument, NULL, OPT_NAME},
 		{"dry-run", no_argument, NULL, OPT_DRY_RUN},
 		{"include-tests", required_argument, NULL, OPT_INCLUDE},
 		{"exclude-tests", required_argument, NULL, OPT_EXCLUDE},
 		{"abort-on-monitored-error", optional_argument, NULL, OPT_ABORT_ON_ERROR},
+		{"disk-usage-limit", required_argument, NULL, OPT_DISK_USAGE_LIMIT},
 		{"sync", no_argument, NULL, OPT_SYNC},
 		{"log-level", required_argument, NULL, OPT_LOG_LEVEL},
 		{"test-list", required_argument, NULL, OPT_TEST_LIST},
@@ -321,6 +392,7 @@ bool parse_options(int argc, char **argv,
 		{"ignore-missing", no_argument, NULL, OPT_IGNORE_MISSING},
 		{"multiple-mode", no_argument, NULL, OPT_MULTIPLE},
 		{"inactivity-timeout", required_argument, NULL, OPT_TIMEOUT},
+		{"per-test-timeout", required_argument, NULL, OPT_PER_TEST_TIMEOUT},
 		{"overall-timeout", required_argument, NULL, OPT_OVERALL_TIMEOUT},
 		{"use-watchdog", no_argument, NULL, OPT_WATCHDOG},
 		{"piglit-style-dmesg", no_argument, NULL, OPT_PIGLIT_DMESG},
@@ -339,6 +411,9 @@ bool parse_options(int argc, char **argv,
 	while ((c = getopt_long(argc, argv, "hn:dt:x:sl:omb:L",
 				long_options, NULL)) != -1) {
 		switch (c) {
+		case OPT_VERSION:
+			print_version();
+			goto error;
 		case OPT_HELP:
 			usage(NULL, stdout);
 			goto error;
@@ -360,6 +435,12 @@ bool parse_options(int argc, char **argv,
 			if (!parse_abort_conditions(settings, optarg))
 				goto error;
 			break;
+		case OPT_DISK_USAGE_LIMIT:
+			if (!parse_usage_limit(settings, optarg)) {
+				usage("Cannot parse disk usage limit", stderr);
+				goto error;
+			}
+			break;
 		case OPT_SYNC:
 			settings->sync = true;
 			break;
@@ -384,6 +465,9 @@ bool parse_options(int argc, char **argv,
 		case OPT_TIMEOUT:
 			settings->inactivity_timeout = atoi(optarg);
 			break;
+		case OPT_PER_TEST_TIMEOUT:
+			settings->per_test_timeout = atoi(optarg);
+			break;
 		case OPT_OVERALL_TIMEOUT:
 			settings->overall_timeout = atoi(optarg);
 			break;
@@ -603,6 +687,7 @@ bool serialize_settings(struct settings *settings)
 	}
 
 	SERIALIZE_LINE(f, settings, abort_mask, "%d");
+	SERIALIZE_LINE(f, settings, disk_usage_limit, "%zd");
 	if (settings->test_list)
 		SERIALIZE_LINE(f, settings, test_list, "%s");
 	if (settings->name)
@@ -613,6 +698,7 @@ bool serialize_settings(struct settings *settings)
 	SERIALIZE_LINE(f, settings, overwrite, "%d");
 	SERIALIZE_LINE(f, settings, multiple_mode, "%d");
 	SERIALIZE_LINE(f, settings, inactivity_timeout, "%d");
+	SERIALIZE_LINE(f, settings, per_test_timeout, "%d");
 	SERIALIZE_LINE(f, settings, overall_timeout, "%d");
 	SERIALIZE_LINE(f, settings, use_watchdog, "%d");
 	SERIALIZE_LINE(f, settings, piglit_style_dmesg, "%d");
@@ -647,9 +733,10 @@ bool read_settings_from_file(struct settings *settings, FILE *f)
 
 	settings->dmesg_warn_level = -1;
 
-	while (fscanf(f, "%ms : %ms", &name, &val) == 2) {
+	while (fscanf(f, "%ms : %m[^\n]", &name, &val) == 2) {
 		int numval = atoi(val);
 		PARSE_LINE(settings, name, val, abort_mask, numval);
+		PARSE_LINE(settings, name, val, disk_usage_limit, strtoul(val, NULL, 10));
 		PARSE_LINE(settings, name, val, test_list, val ? strdup(val) : NULL);
 		PARSE_LINE(settings, name, val, name, val ? strdup(val) : NULL);
 		PARSE_LINE(settings, name, val, dry_run, numval);
@@ -658,6 +745,7 @@ bool read_settings_from_file(struct settings *settings, FILE *f)
 		PARSE_LINE(settings, name, val, overwrite, numval);
 		PARSE_LINE(settings, name, val, multiple_mode, numval);
 		PARSE_LINE(settings, name, val, inactivity_timeout, numval);
+		PARSE_LINE(settings, name, val, per_test_timeout, numval);
 		PARSE_LINE(settings, name, val, overall_timeout, numval);
 		PARSE_LINE(settings, name, val, use_watchdog, numval);
 		PARSE_LINE(settings, name, val, piglit_style_dmesg, numval);
