@@ -94,6 +94,7 @@ static void invalid_tests(int fd)
 
 	f.flags = LOCAL_DRM_MODE_FB_MODIFIERS;
 
+	igt_describe("Test that addfb2 call fails correctly for unused handle");
 	igt_subtest("unused-handle") {
 		igt_require_fb_modifiers(fd);
 
@@ -103,6 +104,7 @@ static void invalid_tests(int fd)
 		f.handles[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused pitches");
 	igt_subtest("unused-pitches") {
 		igt_require_fb_modifiers(fd);
 
@@ -112,6 +114,7 @@ static void invalid_tests(int fd)
 		f.pitches[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused offset");
 	igt_subtest("unused-offsets") {
 		igt_require_fb_modifiers(fd);
 
@@ -121,6 +124,7 @@ static void invalid_tests(int fd)
 		f.offsets[1] = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for unused modifier");
 	igt_subtest("unused-modifier") {
 		igt_require_fb_modifiers(fd);
 
@@ -130,7 +134,9 @@ static void invalid_tests(int fd)
 		f.modifier[1] = 0;
 	}
 
+	igt_describe("Check if addfb2 call works for clobbered modifier");
 	igt_subtest("clobberred-modifier") {
+		igt_require_intel(fd);
 		f.flags = 0;
 		f.modifier[0] = 0;
 		gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4);
@@ -140,6 +146,7 @@ static void invalid_tests(int fd)
 		igt_assert(f.modifier[0] == 0);
 	}
 
+	igt_describe("Check if addfb2 call works for legacy formats");
 	igt_subtest("legacy-format") {
 		struct {
 			/* drm_mode_legacy_fb_format() */
@@ -265,11 +272,13 @@ static void pitch_tests(int fd)
 		igt_assert(gem_bo);
 	}
 
+	igt_describe("Test that addfb2 call fails correctly without handle");
 	igt_subtest("no-handle") {
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
 			   errno == EINVAL);
 	}
 
+	igt_describe("Check if addfb2 call works with given handle");
 	f.handles[0] = gem_bo;
 	igt_subtest("basic") {
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
@@ -277,11 +286,23 @@ static void pitch_tests(int fd)
 		f.fb_id = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for bad-pitches");
 	for (i = 0; i < ARRAY_SIZE(bad_pitches); i++) {
 		igt_subtest_f("bad-pitch-%i", bad_pitches[i]) {
 			f.pitches[0] = bad_pitches[i];
-			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-				   errno == EINVAL);
+			igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f), -1);
+			igt_assert(errno != 0);
+			if (is_i915_device(fd)) {
+				igt_assert_eq(errno, EINVAL);
+			} else if (is_nouveau_device(fd)) {
+				if (bad_pitches[i] > 4 * 1024)
+					igt_assert_eq(errno, ERANGE);
+				else
+					igt_assert_eq(errno, EINVAL);
+			} else {
+				igt_info("Unknown vendor; errno unchecked (returned %i)", errno);
+			}
+			errno = 0;
 		}
 	}
 
@@ -302,6 +323,7 @@ static void tiling_tests(int fd)
 
 	igt_subtest_group {
 		igt_fixture {
+			igt_require_intel(fd);
 			tiled_x_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
 				DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_X_TILED,
 				1024*4, NULL, NULL, NULL);
@@ -318,7 +340,8 @@ static void tiling_tests(int fd)
 		}
 
 		f.pitches[0] = 1024*4;
-		igt_subtest("basic-X-tiled") {
+		igt_describe("Check if addfb2 and rmfb call works for basic x-tiling test");
+		igt_subtest("basic-x-tiled-legacy") {
 			f.handles[0] = tiled_x_bo;
 
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
@@ -326,6 +349,7 @@ static void tiling_tests(int fd)
 			f.fb_id = 0;
 		}
 
+		igt_describe("Check if addfb2 call works for x and y tiling");
 		igt_subtest("framebuffer-vs-set-tiling") {
 			f.handles[0] = gem_bo;
 
@@ -337,7 +361,8 @@ static void tiling_tests(int fd)
 			f.fb_id = 0;
 		}
 
-		f.pitches[0] = 512*4;
+		igt_describe("Test that addfb2 call fails correctly for pitches mismatch");
+			f.pitches[0] = 512*4;
 		igt_subtest("tile-pitch-mismatch") {
 			f.handles[0] = tiled_x_bo;
 
@@ -345,8 +370,9 @@ static void tiling_tests(int fd)
 				   errno == EINVAL);
 		}
 
+		igt_describe("Test that addfb2 call fails correctly for basic y-tiling test");
 		f.pitches[0] = 1024*4;
-		igt_subtest("basic-Y-tiled") {
+		igt_subtest("basic-y-tiled-legacy") {
 			f.handles[0] = tiled_y_bo;
 
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
@@ -365,6 +391,8 @@ static void size_tests(int fd)
 	struct drm_mode_fb_cmd2 f = {};
 	struct drm_mode_fb_cmd2 f_16 = {};
 	struct drm_mode_fb_cmd2 f_8 = {};
+	struct drm_mode_fb_cmd2 *framebuffers[] = {&f, &f_16, &f_8};
+	int i;
 
 	f.width = 1024;
 	f.height = 1024;
@@ -394,6 +422,7 @@ static void size_tests(int fd)
 	f_16.handles[0] = gem_bo;
 	f_8.handles[0] = gem_bo;
 
+	igt_describe("Check if addfb2 call works with max size of  buffer object");
 	igt_subtest("size-max") {
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
@@ -409,6 +438,7 @@ static void size_tests(int fd)
 	f.width++;
 	f_16.width++;
 	f_8.width++;
+	igt_describe("Test that addfb2 call fails correctly with increased width of fb");
 	igt_subtest("too-wide") {
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
 			   errno == EINVAL);
@@ -423,22 +453,38 @@ static void size_tests(int fd)
 	f.height++;
 	f_16.height++;
 	f_8.height++;
+	igt_describe("Test that addfb2 call fails correctly with increased height of fb");
 	igt_subtest("too-high") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_16) == -1 &&
-			   errno == EINVAL);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f_8) == -1 &&
-			   errno == EINVAL);
+		for (i = 0; i < ARRAY_SIZE(framebuffers); i++) {
+			igt_debug("Checking framebuffer %i\n", i);
+			igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, framebuffers[i]), -1);
+			igt_assert(errno != 0);
+			if (is_i915_device(fd))
+				igt_assert_eq(errno, EINVAL);
+			else if (is_nouveau_device(fd))
+				igt_assert_eq(errno, ERANGE);
+			else
+				igt_info("Unknown vendor; errno unchecked (returned %i)", errno);
+			errno = 0;
+		}
 	}
 
+	igt_describe("Test that addfb2 call fails correctly with small size of buffer object");
 	f.handles[0] = gem_bo_small;
 	igt_subtest("bo-too-small") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
+		igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f), -1);
+		igt_assert(errno != 0);
+		if (is_i915_device(fd))
+			igt_assert_eq(errno, EINVAL);
+		else if (is_nouveau_device(fd))
+			igt_assert_eq(errno, ERANGE);
+		else
+			igt_info("Unknown vendor; errno unchecked (returned %i)", errno);
+		errno = 0;
 	}
 
 	/* Just to check that the parameters would work. */
+	igt_describe("Check if addfb2 call works for given height");
 	f.height = 1020;
 	igt_subtest("small-bo") {
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
@@ -446,7 +492,9 @@ static void size_tests(int fd)
 		f.fb_id = 0;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly with small buffer object after changing tile");
 	igt_subtest("bo-too-small-due-to-tiling") {
+		igt_require_intel(fd);
 		gem_set_tiling(fd, gem_bo_small, I915_TILING_X, 1024*4);
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
 			   errno == EINVAL);
@@ -479,6 +527,7 @@ static void addfb25_tests(int fd)
 		f.handles[0] = gem_bo;
 	}
 
+	igt_describe("Test that addfb2 call fails correctly for x-tiling with given modifier");
 	igt_subtest("addfb25-modifier-no-flag") {
 		igt_require_fb_modifiers(fd);
 
@@ -489,6 +538,7 @@ static void addfb25_tests(int fd)
 	igt_fixture
 		f.flags = LOCAL_DRM_MODE_FB_MODIFIERS;
 
+	igt_describe("Test that addfb2 call fails correctly for irrelevant modifier");
 	igt_subtest("addfb25-bad-modifier") {
 		igt_require_fb_modifiers(fd);
 
@@ -498,22 +548,26 @@ static void addfb25_tests(int fd)
 
 	igt_subtest_group {
 		igt_fixture {
+			igt_require_intel(fd);
 			gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
 			igt_require_fb_modifiers(fd);
 		}
 
-		igt_subtest("addfb25-X-tiled-mismatch") {
+		igt_describe("Test that addfb2 call fails correctly for irrelevant x-tiling");
+		igt_subtest("addfb25-x-tiled-mismatch-legacy") {
 			f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
 			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
 		}
 
-		igt_subtest("addfb25-X-tiled") {
+		igt_describe("Check if addfb2 call works for x-tiling");
+		igt_subtest("addfb25-x-tiled-legacy") {
 			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
 			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
 			f.fb_id = 0;
 		}
 
+		igt_describe("Check if addfb2 call works for relevant combination of tiling and fbs");
 		igt_subtest("addfb25-framebuffer-vs-set-tiling") {
 			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
 			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
@@ -527,23 +581,20 @@ static void addfb25_tests(int fd)
 		gem_close(fd, gem_bo);
 }
 
-static int addfb_expected_ret(int fd)
+static int addfb_expected_ret(igt_display_t *display, struct local_drm_mode_fb_cmd2 *f)
 {
-	int gen;
-
-	if (!is_i915_device(fd))
-		return 0;
-
-	gen = intel_gen(intel_get_drm_devid(fd));
-	return gen >= 9 ? 0 : -1;
+	return igt_display_has_format_mod(display, f->pixel_format,
+					  f->modifier[0]) ? 0 : -1;
 }
 
 static void addfb25_ytile(int fd)
 {
 	struct local_drm_mode_fb_cmd2 f = {};
-	int gen;
+	igt_display_t display;
 
 	igt_fixture {
+		igt_display_require(&display, fd);
+
 		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
 			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
@@ -563,37 +614,41 @@ static void addfb25_ytile(int fd)
 		f.handles[0] = gem_bo;
 	}
 
-	igt_subtest("addfb25-Y-tiled") {
+	igt_describe("Check if addfb2 call works for y-tiling");
+	igt_subtest("addfb25-y-tiled-legacy") {
 		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
 		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
-			   addfb_expected_ret(fd));
-		if (!addfb_expected_ret(fd))
+			   addfb_expected_ret(&display, &f));
+		if (!addfb_expected_ret(&display, &f))
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
 		f.fb_id = 0;
 	}
 
-	igt_subtest("addfb25-Yf-tiled") {
+	igt_describe("Check if addfb2 call works for yf-tiling");
+	igt_subtest("addfb25-yf-tiled-legacy") {
 		igt_require_fb_modifiers(fd);
+		igt_require_intel(fd);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Yf_TILED;
 		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
-			   addfb_expected_ret(fd));
-		if (!addfb_expected_ret(fd))
+			   addfb_expected_ret(&display, &f));
+		if (!addfb_expected_ret(&display, &f))
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
 		f.fb_id = 0;
 	}
 
-	igt_subtest("addfb25-Y-tiled-small") {
+	igt_describe("Test that addfb2 call fails correctly for y-tiling with given height and modifier");
+	igt_subtest("addfb25-y-tiled-small-legacy") {
 		igt_require_fb_modifiers(fd);
-
-		gen = intel_gen(intel_get_drm_devid(fd));
-		igt_require(gen >= 9);
+		igt_require_intel(fd);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
 		f.height = 1023;
 		f.handles[0] = gem_bo_small;
+		igt_require(addfb_expected_ret(&display, &f) == 0);
 		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
 		f.fb_id = 0;
 	}
@@ -601,6 +656,7 @@ static void addfb25_ytile(int fd)
 	igt_fixture {
 		gem_close(fd, gem_bo);
 		gem_close(fd, gem_bo_small);
+		igt_display_fini(&display);
 	}
 }
 
@@ -631,6 +687,7 @@ static void prop_tests(int fd)
 	get_props.count_props = 1;
 	get_props.obj_id = f.fb_id;
 
+	igt_describe("Test that get-properties ioctl call fails correctly for invalid object type");
 	igt_subtest("invalid-get-prop-any") {
 		get_props.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
 
@@ -638,6 +695,7 @@ static void prop_tests(int fd)
 				    &get_props) == -1 && errno == EINVAL);
 	}
 
+	igt_describe("Test that get-properties ioctl call fails correctly for fb mode object");
 	igt_subtest("invalid-get-prop") {
 		get_props.obj_type = DRM_MODE_OBJECT_FB;
 
@@ -649,6 +707,7 @@ static void prop_tests(int fd)
 	set_prop.prop_id = 1;
 	set_prop.obj_id = f.fb_id;
 
+	igt_describe("Test that set-properties ioctl call fails correctly for invalid object type");
 	igt_subtest("invalid-set-prop-any") {
 		set_prop.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
 
@@ -656,6 +715,7 @@ static void prop_tests(int fd)
 				    &set_prop) == -1 && errno == EINVAL);
 	}
 
+	igt_describe("Test that get-properties ioctl call fails correctly for fb mode object");
 	igt_subtest("invalid-set-prop") {
 		set_prop.obj_type = DRM_MODE_OBJECT_FB;
 
@@ -687,6 +747,7 @@ static void master_tests(int fd)
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
 	}
 
+	igt_describe("Check that only master can rmfb");
 	igt_subtest("master-rmfb") {
 		int master2_fd;
 
