@@ -39,6 +39,7 @@ struct data {
 	igt_display_t display;
 	struct igt_fb red, green;
 	unsigned int cp_tests;
+	struct udev_monitor *uevent_monitor;
 } data;
 
 /* Test flags */
@@ -112,143 +113,6 @@ static int wait_flip_event(void)
 	return rc;
 }
 
-static bool hdcp_event(struct udev_monitor *uevent_monitor,
-		       struct udev *udev, uint32_t conn_id, uint32_t prop_id)
-{
-	struct udev_device *dev;
-	dev_t udev_devnum;
-	struct stat s;
-	const char *hotplug, *connector, *property;
-	bool ret = false;
-
-	dev = udev_monitor_receive_device(uevent_monitor);
-	if (!dev)
-		goto out;
-
-	udev_devnum = udev_device_get_devnum(dev);
-	fstat(data.display.drm_fd, &s);
-
-	hotplug = udev_device_get_property_value(dev, "HOTPLUG");
-	if (!(memcmp(&s.st_rdev, &udev_devnum, sizeof(dev_t)) == 0 &&
-	    hotplug && atoi(hotplug) == 1)) {
-		igt_debug("Not a Hotplug event\n");
-		goto out_dev;
-	}
-
-	connector = udev_device_get_property_value(dev, "CONNECTOR");
-	if (!(memcmp(&s.st_rdev, &udev_devnum, sizeof(dev_t)) == 0 &&
-	    connector && atoi(connector) == conn_id)) {
-		igt_debug("Not for connector id: %u\n", conn_id);
-		goto out_dev;
-	}
-
-	property = udev_device_get_property_value(dev, "PROPERTY");
-	if (!(memcmp(&s.st_rdev, &udev_devnum, sizeof(dev_t)) == 0 &&
-	    property && atoi(property) == prop_id)) {
-		igt_debug("Not for property id: %u\n", prop_id);
-		goto out_dev;
-	}
-	ret = true;
-
-out_dev:
-	udev_device_unref(dev);
-out:
-	return ret;
-}
-
-static void hdcp_udev_fini(struct udev_monitor *uevent_monitor,
-			   struct udev *udev)
-{
-	if (uevent_monitor)
-		udev_monitor_unref(uevent_monitor);
-	if (udev)
-		udev_unref(udev);
-}
-
-static int hdcp_udev_init(struct udev_monitor **uevent_monitor,
-			  struct udev **udev, int *udev_fd)
-{
-	int ret = -EINVAL;
-
-	*udev = udev_new();
-	if (!*udev) {
-		igt_info("failed to create udev object\n");
-		goto out;
-	}
-
-	*uevent_monitor = udev_monitor_new_from_netlink(*udev, "udev");
-	if (!*uevent_monitor) {
-		igt_info("failed to create udev event monitor\n");
-		goto out;
-	}
-
-	ret = udev_monitor_filter_add_match_subsystem_devtype(*uevent_monitor,
-							      "drm",
-							      "drm_minor");
-	if (ret < 0) {
-		igt_info("failed to filter for drm events\n");
-		goto out;
-	}
-
-	ret = udev_monitor_enable_receiving(*uevent_monitor);
-	if (ret < 0) {
-		igt_info("failed to enable udev event reception\n");
-		goto out;
-	}
-
-	*udev_fd = udev_monitor_get_fd(*uevent_monitor);
-	if (*udev_fd < 0) {
-		igt_info("failed to get udev_fd on uevent monitor\n");
-		ret = *udev_fd;
-		goto out;
-	}
-
-	return ret;
-
-out:
-	hdcp_udev_fini(*uevent_monitor, *udev);
-	return ret;
-}
-
-#define MAX_EVENTS	10
-static bool wait_for_hdcp_event(uint32_t conn_id, uint32_t prop_id,
-				uint32_t timeout_mSec)
-{
-
-	struct udev_monitor *uevent_monitor = NULL;
-	struct udev *udev = NULL;
-	int udev_fd, epoll_fd;
-	struct epoll_event event, events[MAX_EVENTS];
-	bool ret = false;
-
-	if (hdcp_udev_init(&uevent_monitor, &udev, &udev_fd) < 0)
-		return false;
-
-	epoll_fd = epoll_create1(0);
-	if (epoll_fd == -1) {
-		igt_info("Failed to create epoll fd. %d\n", epoll_fd);
-		goto out_ep_create;
-	}
-
-	event.events = EPOLLIN | EPOLLERR;
-	event.data.fd = 0;
-
-	if (epoll_ctl(epoll_fd, EPOLL_CTL_ADD, udev_fd, &event)) {
-		igt_info("failed to fd into epoll\n");
-		goto out_ep_ctl;
-	}
-
-	if (epoll_wait(epoll_fd, events, MAX_EVENTS, timeout_mSec))
-		ret = hdcp_event(uevent_monitor, udev, conn_id, prop_id);
-
-out_ep_ctl:
-	if (close(epoll_fd))
-		igt_info("failed to close the epoll fd\n");
-out_ep_create:
-	hdcp_udev_fini(uevent_monitor, udev);
-	return ret;
-}
-
 static bool
 wait_for_prop_value(igt_output_t *output, uint64_t expected,
 		    uint32_t timeout_mSec)
@@ -257,9 +121,10 @@ wait_for_prop_value(igt_output_t *output, uint64_t expected,
 	int i;
 
 	if (data.cp_tests & CP_UEVENT && expected != CP_UNDESIRED) {
-		igt_assert_f(wait_for_hdcp_event(output->id,
+		igt_assert_f(igt_connector_event_detected(data.uevent_monitor,
+							  output->id,
 			     output->props[IGT_CONNECTOR_CONTENT_PROTECTION],
-			     timeout_mSec), "uevent is not received");
+			     timeout_mSec / 1000), "uevent is not received");
 
 		val = igt_output_get_prop(output,
 					  IGT_CONNECTOR_CONTENT_PROTECTION);
@@ -314,16 +179,10 @@ static void modeset_with_fb(const enum pipe pipe, igt_output_t *output,
 	igt_output_override_mode(output, &mode);
 	igt_output_set_pipe(output, pipe);
 
-	igt_create_color_fb(display->drm_fd, mode.hdisplay, mode.vdisplay,
-			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
-			    1.f, 0.f, 0.f, &data.red);
-	igt_create_color_fb(display->drm_fd, mode.hdisplay, mode.vdisplay,
-			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
-			    0.f, 1.f, 0.f, &data.green);
-
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	igt_display_commit2(display, s);
 	igt_plane_set_fb(primary, &data.red);
+	igt_fb_set_size(&data.red, primary, mode.hdisplay, mode.vdisplay);
 
 	/* Wait for Flip completion before starting the HDCP authentication */
 	commit_display_and_wait_for_flip(s);
@@ -593,6 +452,68 @@ static bool sink_hdcp2_capable(igt_output_t *output)
 	return strstr(buf, "HDCP2.2");
 }
 
+static void prepare_modeset_on_mst_output(igt_output_t *output, enum pipe pipe)
+{
+	drmModeConnectorPtr c = output->config.connector;
+	drmModeModeInfo *mode;
+	igt_plane_t *primary;
+	int i, width, height;
+
+	mode = igt_output_get_mode(output);
+
+	/*
+	 * TODO: Add logic to use the highest possible modes on each output.
+	 * Currently using 2k modes by default on all the outputs.
+	 */
+	igt_debug("Before mode override: Output %s Mode hdisplay %d Mode vdisplay %d\n",
+		   output->name, mode->hdisplay, mode->vdisplay);
+
+	if (mode->hdisplay > 1920 && mode->vdisplay > 1080) {
+		for (i = 0; i < c->count_modes; i++) {
+			if (c->modes[i].hdisplay <= 1920 && c->modes[i].vdisplay <= 1080) {
+				mode = &c->modes[i];
+				igt_output_override_mode(output, mode);
+				break;
+			}
+		}
+	}
+
+	igt_debug("After mode overide: Output %s Mode hdisplay %d Mode vdisplay %d\n",
+		   output->name, mode->hdisplay, mode->vdisplay);
+
+	width = mode->hdisplay;
+	height = mode->vdisplay;
+
+	igt_output_set_pipe(output, pipe);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+	igt_plane_set_fb(primary, NULL);
+	igt_plane_set_fb(primary, pipe % 2 ? &data.red : &data.green);
+	igt_fb_set_size(pipe % 2 ? &data.red : &data.green, primary, width, height);
+	igt_plane_set_size(primary, width, height);
+}
+
+static bool output_hdcp_capable(igt_output_t *output, int content_type)
+{
+		if (!output->props[IGT_CONNECTOR_CONTENT_PROTECTION])
+			return false;
+
+		if (!output->props[IGT_CONNECTOR_HDCP_CONTENT_TYPE] &&
+		    content_type)
+			return false;
+
+		if (content_type && !sink_hdcp2_capable(output)) {
+			igt_info("\tSkip %s (Sink has no HDCP2.2 support)\n",
+				 output->name);
+			return false;
+		} else if (!sink_hdcp_capable(output)) {
+			igt_info("\tSkip %s (Sink has no HDCP support)\n",
+				 output->name);
+			return false;
+		}
+
+		return true;
+}
+
 static void
 test_content_protection(enum igt_commit_style s, int content_type)
 {
@@ -605,25 +526,9 @@ test_content_protection(enum igt_commit_style s, int content_type)
 			      "mei_hdcp module is not loaded\n");
 
 	for_each_connected_output(display, output) {
-		if (!output->props[IGT_CONNECTOR_CONTENT_PROTECTION])
+		if (!output_hdcp_capable(output, content_type))
 			continue;
 
-		if (!output->props[IGT_CONNECTOR_HDCP_CONTENT_TYPE] &&
-		    content_type)
-			continue;
-
-		igt_info("CP Test execution on %s\n", output->name);
-
-		if (content_type && !sink_hdcp2_capable(output)) {
-			igt_info("\tSkip %s (Sink has no HDCP2.2 support)\n",
-				 output->name);
-			continue;
-		} else if (!sink_hdcp_capable(output)) {
-			igt_info("\tSkip %s (Sink has no HDCP support)\n",
-				 output->name);
-			continue;
-		}
-
 		test_content_protection_on_output(output, s, content_type);
 		valid_tests++;
 	}
@@ -631,6 +536,151 @@ test_content_protection(enum igt_commit_style s, int content_type)
 	igt_require_f(valid_tests, "No connector found with HDCP capability\n");
 }
 
+static int parse_path_blob(char *blob_data)
+{
+	int connector_id;
+	char *encoder;
+
+	encoder = strtok(blob_data, ":");
+	igt_assert_f(!strcmp(encoder, "mst"), "PATH connector property expected to have 'mst'\n");
+
+	connector_id = atoi(strtok(NULL, "-"));
+
+	return connector_id;
+}
+
+static bool output_is_dp_mst(igt_output_t *output, int i)
+{
+	drmModePropertyBlobPtr path_blob = NULL;
+	uint64_t path_blob_id;
+	drmModeConnector *connector = output->config.connector;
+	struct kmstest_connector_config config;
+	const char *encoder;
+	int connector_id;
+	static int prev_connector_id;
+
+	kmstest_get_connector_config(data.drm_fd, output->config.connector->connector_id, -1, &config);
+	encoder = kmstest_encoder_type_str(config.encoder->encoder_type);
+
+	if (strcmp(encoder, "DP MST"))
+		return false;
+
+	igt_assert(kmstest_get_property(data.drm_fd, connector->connector_id,
+		   DRM_MODE_OBJECT_CONNECTOR, "PATH", NULL,
+		   &path_blob_id, NULL));
+
+	igt_assert(path_blob = drmModeGetPropertyBlob(data.drm_fd, path_blob_id));
+
+	connector_id = parse_path_blob((char *) path_blob->data);
+
+	/*
+	 * Discarding outputs of other DP MST topology.
+	 * Testing only on outputs on the topology we got previously
+	 */
+	if (i == 0) {
+		prev_connector_id = connector_id;
+	} else {
+		if (connector_id != prev_connector_id)
+			return false;
+	}
+
+	drmModeFreePropertyBlob(path_blob);
+
+	return true;
+}
+
+static void test_cp_lic_on_mst(igt_output_t *mst_outputs[], int valid_outputs, bool first_output)
+{
+	int ret, count;
+	uint64_t val;
+
+	/* Only wait for the first output, this optimizes the test execution time */
+	ret = wait_for_prop_value(mst_outputs[first_output], CP_DESIRED, LIC_PERIOD_MSEC);
+	igt_assert_f(!ret, "Content Protection LIC Failed on %s\n", mst_outputs[0]->name);
+
+	for (count = first_output + 1; count < valid_outputs; count++) {
+		val = igt_output_get_prop(mst_outputs[count], IGT_CONNECTOR_CONTENT_PROTECTION);
+		igt_assert_f(val != CP_DESIRED, "Content Protection LIC Failed on %s\n", mst_outputs[count]->name);
+	}
+}
+
+static void
+test_content_protection_mst(int content_type)
+{
+	igt_display_t *display = &data.display;
+	igt_output_t *output;
+	int valid_outputs = 0, dp_mst_outputs = 0, ret, count, max_pipe = 0, i;
+	enum pipe pipe;
+	igt_output_t *mst_output[IGT_MAX_PIPES], *hdcp_mst_output[IGT_MAX_PIPES];
+
+	for_each_pipe(display, pipe)
+		max_pipe++;
+
+	pipe = PIPE_A;
+
+	for_each_connected_output(display, output) {
+		if (!output_is_dp_mst(output, dp_mst_outputs))
+			continue;
+
+		igt_assert_f(igt_pipe_connector_valid(pipe, output), "Output-pipe combination invalid\n");
+
+		prepare_modeset_on_mst_output(output, pipe);
+		mst_output[dp_mst_outputs++] = output;
+
+		pipe++;
+
+		if (pipe > max_pipe)
+			break;
+	}
+
+	igt_require_f(dp_mst_outputs > 1, "No DP MST set up with >= 2 outputs found in a single topology\n");
+
+	ret = igt_display_try_commit2(display, COMMIT_ATOMIC);
+	igt_require_f(ret == 0, "Commit failure during MST modeset\n");
+
+	for (count = 0; count < dp_mst_outputs; count++) {
+		if (!output_hdcp_capable(mst_output[count], content_type))
+			continue;
+
+		hdcp_mst_output[valid_outputs++] = mst_output[count];
+	}
+
+	igt_require_f(valid_outputs > 1, "DP MST outputs do not have the required HDCP support\n");
+
+	for (count = 0; count < valid_outputs; count++) {
+		igt_output_set_prop_value(hdcp_mst_output[count], IGT_CONNECTOR_CONTENT_PROTECTION, CP_DESIRED);
+
+		if (output->props[IGT_CONNECTOR_HDCP_CONTENT_TYPE])
+			igt_output_set_prop_value(hdcp_mst_output[count], IGT_CONNECTOR_HDCP_CONTENT_TYPE, content_type);
+	}
+
+	igt_display_commit2(display, COMMIT_ATOMIC);
+
+	for (count = 0; count < valid_outputs; count++) {
+		ret = wait_for_prop_value(hdcp_mst_output[count], CP_ENABLED, KERNEL_AUTH_TIME_ALLOWED_MSEC);
+		igt_assert_f(ret, "Content Protection not enabled on %s\n", hdcp_mst_output[count]->name);
+	}
+
+	if (data.cp_tests & CP_LIC)
+		test_cp_lic_on_mst(hdcp_mst_output, valid_outputs, 0);
+
+	/*
+	 * Verify if CP is still enabled on other outputs by disabling CP on the first output.
+	 */
+	igt_debug("CP Prop being UNDESIRED on %s\n", hdcp_mst_output[0]->name);
+	test_cp_disable(hdcp_mst_output[0], COMMIT_ATOMIC);
+
+	/* CP is expected to be still enabled on other outputs*/
+	for (i = 1; i < valid_outputs; i++) {
+		/* Wait for the timeout to verify CP is not disabled */
+		ret = wait_for_prop_value(hdcp_mst_output[i], CP_UNDESIRED, KERNEL_DISABLE_TIME_ALLOWED_MSEC);
+		igt_assert_f(!ret, "Content Protection not enabled on %s\n", hdcp_mst_output[i]->name);
+	}
+
+	if (data.cp_tests & CP_LIC)
+		test_cp_lic_on_mst(hdcp_mst_output, valid_outputs, 1);
+}
+
 static void test_content_protection_cleanup(void)
 {
 	igt_display_t *display = &data.display;
@@ -649,60 +699,101 @@ static void test_content_protection_cleanup(void)
 		igt_info("CP Prop being UNDESIRED on %s\n", output->name);
 		test_cp_disable(output, COMMIT_ATOMIC);
 	}
+
+	igt_remove_fb(data.drm_fd, &data.red);
+	igt_remove_fb(data.drm_fd, &data.green);
+}
+
+static void create_fbs(void)
+{
+	igt_output_t *output;
+	int width = 0, height = 0;
+	drmModeModeInfo *mode;
+
+	for_each_connected_output(&data.display, output) {
+		mode = igt_output_get_mode(output);
+		igt_assert(mode);
+
+		width = max(width, mode->hdisplay);
+		height = max(height, mode->vdisplay);
+	}
+
+	igt_create_color_fb(data.drm_fd, width, height,
+			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    1.f, 0.f, 0.f, &data.red);
+	igt_create_color_fb(data.drm_fd, width, height,
+			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    0.f, 1.f, 0.f, &data.green);
 }
 
 igt_main
 {
 	igt_fixture {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
-
 		igt_display_require(&data.display, data.drm_fd);
+		igt_display_require_output(&data.display);
+		create_fbs();
 	}
 
+	igt_describe("Test content protection with legacy style commit.");
 	igt_subtest("legacy") {
 		data.cp_tests = 0;
 		test_content_protection(COMMIT_LEGACY, HDCP_CONTENT_TYPE_0);
 	}
 
+	igt_describe("Test content protection with atomic modesetting");
 	igt_subtest("atomic") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = 0;
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_0);
 	}
 
+	igt_describe("Test content protection with DPMS ON/OFF during atomic modesetting.");
 	igt_subtest("atomic-dpms") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = CP_DPMS;
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_0);
 	}
 
+	igt_describe("Test for the integrity of link.");
 	igt_subtest("LIC") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = CP_LIC;
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_0);
 	}
 
+	igt_describe("Test content protection with content type 1 that "
+		     "can be handled only through HDCP2.2.");
 	igt_subtest("type1") {
 		igt_require(data.display.is_atomic);
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_1);
 	}
 
+	igt_describe("Test the teardown and rebuild of the interface between "
+		     "I915 and mei hdcp.");
 	igt_subtest("mei_interface") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = CP_MEI_RELOAD;
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_1);
 	}
 
+	igt_describe("Test the content type change when the content protection already "
+		     "enabled.");
 	igt_subtest("content_type_change") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = CP_TYPE_CHANGE;
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_1);
 	}
 
+	igt_describe("Test to detect the HDCP status change when we are reading the uevent "
+		     "sent with the corresponding connector id and property id.");
 	igt_subtest("uevent") {
 		igt_require(data.display.is_atomic);
 		data.cp_tests = CP_UEVENT;
+		data.uevent_monitor = igt_watch_uevents();
+		igt_flush_uevents(data.uevent_monitor);
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_0);
+		igt_cleanup_uevents(data.uevent_monitor);
 	}
 
 	/*
@@ -713,6 +804,8 @@ igt_main
 	 *  either of these options, we test SRM writing from userspace and
 	 *  validation of the same at kernel. Something is better than nothing.
 	 */
+	igt_describe("This test writes the facsimile SRM into the /lib/firmware/ "
+		     "and check the kernel parsing of it by invoking the hdcp authentication.");
 	igt_subtest("srm") {
 		bool ret;
 
@@ -724,6 +817,28 @@ igt_main
 		test_content_protection(COMMIT_ATOMIC, HDCP_CONTENT_TYPE_0);
 	}
 
+	igt_describe("Test Content protection over DP MST");
+	igt_subtest("dp-mst-type-0") {
+		test_content_protection_mst(HDCP_CONTENT_TYPE_0);
+	}
+
+	igt_describe("Test Content protection over DP MST with LIC");
+	igt_subtest("dp-mst-lic-type-0") {
+		data.cp_tests = CP_LIC;
+		test_content_protection_mst(HDCP_CONTENT_TYPE_0);
+	}
+
+	igt_describe("Test Content protection over DP MST");
+	igt_subtest("dp-mst-type-1") {
+		test_content_protection_mst(HDCP_CONTENT_TYPE_1);
+	}
+
+	igt_describe("Test Content protection over DP MST with LIC");
+	igt_subtest("dp-mst-lic-type-1") {
+		data.cp_tests = CP_LIC;
+		test_content_protection_mst(HDCP_CONTENT_TYPE_1);
+	}
+
 	igt_fixture {
 		test_content_protection_cleanup();
 		igt_display_fini(&data.display);
