@@ -30,7 +30,6 @@
 #include <string.h>
 #include <sys/time.h>
 #include <math.h>
-#include "intel_bufmgr.h"
 
 #define MAX_CONNECTORS  10
 #define MAX_CRTCS       6
@@ -38,10 +37,14 @@
 /* max combinations with repetitions */
 #define MAX_COMBINATION_ELEMS   MAX_CRTCS
 
+/* restricted pipe count */
+#define CRTC_RESTRICT_CNT 2
+
 static int drm_fd;
 static drmModeRes *drm_resources;
 static int filter_test_id;
 static bool dry_run;
+static bool all_pipes = false;
 
 const drmModeModeInfo mode_640_480 = {
 	.name		= "640x480",
@@ -161,7 +164,7 @@ static int paint_fb(struct igt_fb *fb, const char *test_name,
 					  crtc_str[i]);
 	}
 
-	igt_put_cairo_ctx(drm_fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 
 	return 0;
 }
@@ -256,7 +259,8 @@ static void get_crtc_config_str(struct crtc_config *crtc, char *buf,
 	}
 }
 
-static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
+static void setup_crtcs(const struct test_config *tconf,
+			struct connector_config *cconf,
 			int connector_count, struct crtc_config *crtcs,
 			int *crtc_count_ret, bool *config_valid_ret)
 {
@@ -264,6 +268,7 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 	int crtc_count;
 	bool config_valid;
 	int i;
+	drmModeRes *resources = tconf->resources;
 	int encoder_usage_count[resources->count_encoders];
 
 	kmstest_unset_all_crtcs(drm_fd, resources);
@@ -352,7 +357,8 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 		drmModeFreeEncoder(encoder);
 	}
 	for (i = 0; i < resources->count_encoders; i++)
-		if (encoder_usage_count[i] > 1)
+		if (encoder_usage_count[i] > 1 &&
+				!!(tconf->flags & TEST_SINGLE_CRTC_CLONE))
 			config_valid = false;
 
 	*crtc_count_ret = crtc_count;
@@ -494,7 +500,7 @@ static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
 	 * which depend on timing, giving the lowest acceptable MTBF of 5.6s
 	 * for 60Hz sampling rate.
 	 */
-	accuracy = 3. * stddev;
+	accuracy = 3. * igt_stats_get_std_error(&stats);
 
 	igt_info("Expected frametime: %.0fus; measured %.1fus +- %.3fus accuracy %.2f%% [%.2f scanlines]\n",
 		 expected, mean, stddev,
@@ -526,10 +532,12 @@ static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
 	 * See:
 	 * https://en.wikipedia.org/wiki/Standard_deviation#Rules_for_normally_distributed_data
 	 */
-	igt_assert_f(fabs(mean - expected) < 1.718 * stddev,
-		     "vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma)\n",
+	igt_assert_f(fabs(mean - expected) < max(line_time(kmode), 1.718 * stddev),
+		     "vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma, %.1f scanlines)\n",
 		     expected, mean, stddev,
-		     fabs(mean - expected), fabs(mean - expected) / stddev);
+		     fabs(mean - expected),
+		     fabs(mean - expected) / stddev,
+		     fabs(mean - expected) / line_time(kmode));
 }
 
 static void test_crtc_config(const struct test_config *tconf,
@@ -603,7 +611,7 @@ static void test_one_combination(const struct test_config *tconf,
 	int crtc_count;
 	bool config_valid;
 
-	setup_crtcs(tconf->resources, cconfs, connector_count, crtcs,
+	setup_crtcs(tconf, cconfs, connector_count, crtcs,
 		    &crtc_count, &config_valid);
 
 	if (config_valid == !(tconf->flags & TEST_INVALID))
@@ -743,6 +751,14 @@ static void get_combinations(int n, int k, bool allow_repetitions,
 	iterate_combinations(n, k, allow_repetitions, 0, 0, &comb, set);
 }
 
+static int get_limit_crtc_count(int count_crtcs, bool all_pipe)
+{
+	if ((count_crtcs <= CRTC_RESTRICT_CNT) || all_pipe)
+		return count_crtcs;
+	else
+		return CRTC_RESTRICT_CNT;
+}
+
 static void test_combinations(const struct test_config *tconf,
 			      int connector_count)
 {
@@ -750,6 +766,7 @@ static void test_combinations(const struct test_config *tconf,
 	struct combination_set crtc_combs;
 	struct connector_config *cconfs;
 	int i;
+	int crtc_count = get_limit_crtc_count(tconf->resources->count_crtcs, all_pipes);
 
 	if (connector_count > 2 && (tconf->flags & TEST_STEALING))
 		return;
@@ -757,20 +774,19 @@ static void test_combinations(const struct test_config *tconf,
 	igt_assert(tconf->resources);
 
 	connector_combs.capacity = pow(tconf->resources->count_connectors,
-				       tconf->resources->count_crtcs + 1);
-	crtc_combs.capacity = pow(tconf->resources->count_crtcs,
-				  tconf->resources->count_crtcs + 1);
-
+				       crtc_count + 1);
+	crtc_combs.capacity = pow(crtc_count,
+				  crtc_count + 1);
 	connector_combs.items = malloc(connector_combs.capacity * sizeof(struct combination));
 	crtc_combs.items = malloc(crtc_combs.capacity * sizeof(struct combination));
 
 	get_combinations(tconf->resources->count_connectors, connector_count,
 			 false, &connector_combs);
-	get_combinations(tconf->resources->count_crtcs, connector_count,
-			 true, &crtc_combs);
+	get_combinations(crtc_count, connector_count, true, &crtc_combs);
 
 	igt_info("Testing: %s %d connector combinations\n", tconf->name,
 		 connector_count);
+
 	for (i = 0; i < connector_combs.count; i++) {
 		int *connector_idxs;
 		int ret;
@@ -808,9 +824,10 @@ free_cconfs:
 static void run_test(const struct test_config *tconf)
 {
 	int connector_num;
+	int crtc_count = get_limit_crtc_count(tconf->resources->count_crtcs, all_pipes);
 
 	connector_num = tconf->flags & TEST_CLONE ? 2 : 1;
-	for (; connector_num <= tconf->resources->count_crtcs; connector_num++)
+	for (; connector_num <= crtc_count; connector_num++)
 		test_combinations(tconf, connector_num);
 }
 
@@ -820,6 +837,9 @@ static int opt_handler(int opt, int opt_index, void *data)
 	case 'd':
 		dry_run = true;
 		break;
+	case 'e':
+		all_pipes = true;
+		break;
 	case 't':
 		filter_test_id = atoi(optarg);
 		break;
@@ -832,9 +852,10 @@ static int opt_handler(int opt, int opt_index, void *data)
 
 const char *help_str =
 	"  -d\t\tDon't run any test, only print what would be done. (still needs DRM access)\n"
-	"  -t <test id>\tRun only the test with this id.";
+	"  -t <test id>\tRun only the test with this id\n"
+	"  -e \t\tRun on all pipes. (Default it will Run only two pipes)\n";
 
-igt_main_args("dt:", NULL, help_str, opt_handler, NULL)
+igt_main_args("det:", NULL, help_str, opt_handler, NULL)
 {
 	const struct {
 		enum test_flags flags;
@@ -854,8 +875,6 @@ igt_main_args("dt:", NULL, help_str, opt_handler, NULL)
 	};
 	int i;
 
-	igt_skip_on_simulation();
-
 	igt_assert_f(!(dry_run && filter_test_id),
 		     "only one of -d and -t is accepted\n");
 
@@ -869,6 +888,7 @@ igt_main_args("dt:", NULL, help_str, opt_handler, NULL)
 	}
 
 	for (i = 0; i < ARRAY_SIZE(tests); i++) {
+		igt_describe("Tests the mode by iterating through all valid/invalid crtc/connector combinations");
 		igt_subtest(tests[i].name) {
 			struct test_config tconf = {
 				.flags		= tests[i].flags,
