@@ -229,6 +229,8 @@ static void dump_backlight_info(struct context *context,
 {
 	const struct bdb_lfp_backlight_data *backlight = block->data;
 	const struct lfp_backlight_data_entry *blc;
+	const struct lfp_backlight_control_method *control;
+	int i;
 
 	if (sizeof(*blc) != backlight->entry_size) {
 		printf("\tBacklight struct sizes don't match (expected %zu, got %u), skipping\n",
@@ -236,12 +238,27 @@ static void dump_backlight_info(struct context *context,
 		return;
 	}
 
-	blc = &backlight->data[context->panel_type];
+	for (i = 0; i < ARRAY_SIZE(backlight->data); i++) {
+		if (i != context->panel_type && !context->dump_all_panel_types)
+			continue;
 
-	printf("\tInverter type: %d\n", blc->type);
-	printf("\t     polarity: %d\n", blc->active_low_pwm);
-	printf("\t     PWM freq: %d\n", blc->pwm_freq_hz);
-	printf("\tMinimum brightness: %d\n", blc->min_brightness);
+		printf("\tPanel %d%s\n", i,
+		       context->panel_type == i ? " *" : "");
+
+		blc = &backlight->data[i];
+
+		printf("\t\tInverter type: %u\n", blc->type);
+		printf("\t\tActive low: %u\n", blc->active_low_pwm);
+		printf("\t\tPWM freq: %u\n", blc->pwm_freq_hz);
+		printf("\t\tMinimum brightness: %u\n", blc->min_brightness);
+
+		printf("\t\tLevel: %u\n", backlight->level[i]);
+
+		control = &backlight->backlight_control[i];
+
+		printf("\t\tControl type: %u\n", control->type);
+		printf("\t\tController: %u\n", control->controller);
+	}
 }
 
 static const struct {
@@ -944,11 +961,13 @@ static void dump_psr(struct context *context,
 {
 	const struct bdb_psr *psr_block = block->data;
 	int i;
+	uint32_t psr2_tp_time;
 
 	/* The same block ID was used for something else before? */
 	if (context->bdb->version < 165)
 		return;
 
+	psr2_tp_time = psr_block->psr2_tp2_tp3_wakeup_time;
 	for (i = 0; i < 16; i++) {
 		const struct psr_table *psr = &psr_block->psr_table[i];
 
@@ -987,6 +1006,15 @@ static void dump_psr(struct context *context,
 		printf("\t\tTP2/TP3 wakeup time: %d usec (0x%x)\n",
 		       psr->tp2_tp3_wakeup_time * 100,
 		       psr->tp2_tp3_wakeup_time);
+
+		if (context->bdb->version >= 226) {
+			int index;
+			static const uint16_t psr2_tp_times[] = {500, 100, 2500, 5};
+
+			index = (psr2_tp_time >> (i * 2)) & 0x3;
+			printf("\t\tPSR2 TP2/TP3 wakeup time: %d usec (0x%x)\n",
+			       psr2_tp_times[index], index);
+		}
 	}
 }
 
@@ -1520,6 +1548,88 @@ static void dump_mipi_sequence(struct context *context,
 			dump_sequence(sequence_ptrs[i], sequence->version);
 }
 
+#define KB(x) ((x) * 1024)
+
+static int dsc_buffer_block_size(u8 buffer_block_size)
+{
+	switch (buffer_block_size) {
+	case VBT_RC_BUFFER_BLOCK_SIZE_1KB:
+		return KB(1);
+		break;
+	case VBT_RC_BUFFER_BLOCK_SIZE_4KB:
+		return KB(4);
+		break;
+	case VBT_RC_BUFFER_BLOCK_SIZE_16KB:
+		return KB(16);
+		break;
+	case VBT_RC_BUFFER_BLOCK_SIZE_64KB:
+		return KB(64);
+		break;
+	default:
+		return 0;
+	}
+}
+
+static int actual_buffer_size(u8 buffer_block_size, u8 rc_buffer_size)
+{
+	return dsc_buffer_block_size(buffer_block_size) * (rc_buffer_size + 1);
+}
+
+static const char *dsc_max_bpp(u8 value)
+{
+	switch (value) {
+	case 0:
+		return "6";
+	case 1:
+		return "8";
+	case 2:
+		return "10";
+	case 3:
+		return "12";
+	default:
+		return "<unknown>";
+	}
+}
+
+static void dump_compression_parameters(struct context *context,
+					const struct bdb_block *block)
+{
+	const struct bdb_compression_parameters *dsc = block->data;
+	const struct dsc_compression_parameters_entry *data;
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(dsc->data); i++) {
+		/* FIXME: need to handle sizeof(*data) != dsc->entry_size */
+		data = &dsc->data[i];
+
+		if (i != context->panel_type && !context->dump_all_panel_types)
+			continue;
+
+		printf("\tDSC block %d%s\n", i,
+		       i == context->panel_type ? " *" : "");
+		printf("\t\tDSC version: %u.%u\n", data->version_major,
+		       data->version_minor);
+		printf("\t\tActual buffer size: %d\n",
+		       actual_buffer_size(data->rc_buffer_block_size,
+					  data->rc_buffer_size));
+		printf("\t\t\tRC buffer block size: %d (%u)\n",
+		       dsc_buffer_block_size(data->rc_buffer_block_size),
+		       data->rc_buffer_block_size);
+		printf("\t\t\tRC buffer size: %u\n", data->rc_buffer_size);
+		printf("\t\tSlices per line: 0x%02x\n", data->slices_per_line);
+		printf("\t\tLine buffer depth: %u bits (%u)\n",
+		       data->line_buffer_depth + 8, data->line_buffer_depth);
+		printf("\t\tBlock prediction enable: %u\n",
+		       data->block_prediction_enable);
+		printf("\t\tMax bpp: %s bpp (%u)\n", dsc_max_bpp(data->max_bpp),
+		       data->max_bpp);
+		printf("\t\tSupport 8 bpc: %u\n", data->support_8bpc);
+		printf("\t\tSupport 10 bpc: %u\n", data->support_10bpc);
+		printf("\t\tSupport 12 bpc: %u\n", data->support_12bpc);
+		printf("\t\tSlice height: %u\n", data->slice_height);
+	}
+}
+
 /* get panel type from lvds options block, or -1 if block not found */
 static int get_panel_type(struct context *context)
 {
@@ -1637,6 +1747,11 @@ struct dumper dumpers[] = {
 		.name = "MIPI sequence block",
 		.dump = dump_mipi_sequence,
 	},
+	{
+		.id = BDB_COMPRESSION_PARAMETERS,
+		.name = "Compression parameters block",
+		.dump = dump_compression_parameters,
+	},
 };
 
 static void hex_dump(const void *data, uint32_t size)
